# Tamito KAJIYAMA <4 January 2003>
# $Id: calendar.py,v 1.3 2004/05/10 19:24:58 kajiyama Exp $

import time

from Tkinter import *

from Sketch import Translation, SolidPattern, StandardColors, GetFont
from Sketch.Graphics import text, group
from Sketch.Graphics.properties import DefaultTextProperties
from Sketch.UI.sketchdlg import SKModal
from Sketch.UI.miniscroll import MiniScroller
import Sketch.config

NUM_DAYS = [None, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31]

def get_day_of_week(y, m, d):
    # calculate day of the week using a variant of Zeller's Congruence
    # (0 = Sunday, 1 = Monday, ..., 6 = Saturday)
    if m < 3:
        y = y - 1
        m = m + 12
    return (y + y / 4 - y / 100 + y / 400 + (13 * m + 8) / 5 + d) % 7

class Layout:
    def __init__(self, year, month, size, hspace, vspace, align):
        self.year = year
        self.month = month
        self.size = size
        self.hspace = hspace
        self.vspace = vspace
        self.halign = align
        self.valign = text.ALIGN_BASE
        leap = (year % 4 == 0 and year % 100 != 0 or year % 400 == 0)
        if month == 2 and leap:
            self.num_days = 29
        else:
            self.num_days = NUM_DAYS[month]
    def create(self):
        return []

class GridSundayLayout(Layout):
    day_of_week = ["S", "M", "T", "W", "T", "F", "S"]
    def create(self):
        offset = get_day_of_week(self.year, self.month, 1)
        buffer = []
        for n in range(7):
            trafo = Translation((n * self.hspace, self.vspace))
            pattern = self.get_column_color(n)
            object = text.SimpleText(trafo=trafo, text=self.day_of_week[n],
                                     properties=DefaultTextProperties(),
                                     halign=self.halign, valign=self.valign)
            object.SetProperties(fill_pattern=pattern, font_size=self.size)
            buffer.append(object)
        for n in range(self.num_days):
            row, column = divmod(offset + n, 7)
            trafo = Translation((column * self.hspace, - row * self.vspace))
            pattern = self.get_column_color(column)
            object = text.SimpleText(trafo=trafo, text=str(n + 1),
                                     properties=DefaultTextProperties(),
                                     halign=self.halign, valign=self.valign)
            object.SetProperties(fill_pattern=pattern, font_size=self.size)
            buffer.append(object)
        return buffer
    def get_column_color(self, column):
        if column == 0:
            return SolidPattern(StandardColors.red)
        elif column == 6:
            return SolidPattern(StandardColors.blue)
        else:
            return SolidPattern(StandardColors.black)

class GridMondayLayout(Layout):
    day_of_week = ["M", "T", "W", "T", "F", "S", "S"]
    def create(self):
        offset = get_day_of_week(self.year, self.month, 1)
        if offset > 0:
            offset = offset - 1
        else:
            offset = 6
        buffer = []
        for n in range(7):
            trafo = Translation((n * self.hspace, self.vspace))
            pattern = self.get_column_color(n)
            object = text.SimpleText(trafo=trafo, text=self.day_of_week[n],
                                     properties=DefaultTextProperties(),
                                     halign=self.halign, valign=self.valign)
            object.SetProperties(fill_pattern=pattern, font_size=self.size)
            buffer.append(object)
        for n in range(self.num_days):
            row, column = divmod(offset + n, 7)
            trafo = Translation((column * self.hspace, - row * self.vspace))
            pattern = self.get_column_color(column)
            object = text.SimpleText(trafo=trafo, text=str(n + 1),
                                     properties=DefaultTextProperties(),
                                     halign=self.halign, valign=self.valign)
            object.SetProperties(fill_pattern=pattern, font_size=self.size)
            buffer.append(object)
        return buffer
    def get_column_color(self, column):
        if column == 6:
            return SolidPattern(StandardColors.red)
        elif column == 5:
            return SolidPattern(StandardColors.blue)
        else:
            return SolidPattern(StandardColors.black)

class HorizontalLayout(Layout):
    def create(self):
        buffer = []
        for n in range(self.num_days):
            trafo = Translation((n * self.hspace, 0))
            day_of_week = get_day_of_week(self.year, self.month, n + 1)
            if day_of_week == 0:
                pattern = SolidPattern(StandardColors.red)
            elif day_of_week == 6:
                pattern = SolidPattern(StandardColors.blue)
            else:
                pattern = SolidPattern(StandardColors.black)
            object = text.SimpleText(trafo=trafo, text=str(n + 1),
                                     properties=DefaultTextProperties(),
                                     halign=self.halign, valign=self.valign)
            object.SetProperties(fill_pattern=pattern, font_size=self.size)
            buffer.append(object)
        return buffer

class VerticalLayout(Layout):
    def create(self):
        buffer = []
        for n in range(self.num_days):
            trafo = Translation((0, - n * self.vspace))
            day_of_week = get_day_of_week(self.year, self.month, n + 1)
            if day_of_week == 0:
                pattern = SolidPattern(StandardColors.red)
            elif day_of_week == 6:
                pattern = SolidPattern(StandardColors.blue)
            else:
                pattern = SolidPattern(StandardColors.black)
            object = text.SimpleText(trafo=trafo, text=str(n + 1),
                                     properties=DefaultTextProperties(),
                                     halign=self.halign, valign=self.valign)
            object.SetProperties(fill_pattern=pattern, font_size=self.size)
            buffer.append(object)
        return buffer

LAYOUTS = [
    ("Grid (Sunday)", GridSundayLayout),
    ("Grid (Monday)", GridMondayLayout),
    ("Horizontal",    HorizontalLayout),
    ("Vertical",      VerticalLayout),
    ]

ALIGNMENTS = [
    ("Left"  , text.ALIGN_LEFT),
    ("Center", text.ALIGN_CENTER),
    ("Right",  text.ALIGN_RIGHT),
    ]

now = time.localtime(time.time())
params = (LAYOUTS[0][0], now[0], now[1], 14.0, 20.0, 20.0, ALIGNMENTS[2][0])

class ParameterDialog(SKModal):
    title = "Calendar"
    def build_dlg(self):
        self.layout = StringVar(self.top)
        self.layout.set(params[0])
        self.year = IntVar(self.top)
        self.year.set(params[1])
        self.month = IntVar(self.top)
        self.month.set(params[2])
        self.size = DoubleVar(self.top)
        self.size.set(params[3])
        self.hspace = DoubleVar(self.top)
        self.hspace.set(params[4])
        self.vspace = DoubleVar(self.top)
        self.vspace.set(params[5])
        self.align = StringVar(self.top)
        self.align.set(params[6])
        frame = Frame(self.top)
        frame.pack(fill=X)
        label = Label(frame, text="Year", width=6)
        label.pack(side=LEFT)
        entry = Entry(frame, textvariable=self.year, width=6, justify=RIGHT)
        entry.pack(side=LEFT)
        scroll = MiniScroller(frame, variable=self.year, min=0)
        scroll.pack(side=LEFT, fill=Y)
        frame = Frame(self.top)
        frame.pack(fill=X)
        label = Label(frame, text="Month", width=6)
        label.pack(side=LEFT)
        entry = Entry(frame, textvariable=self.month, width=6, justify=RIGHT)
        entry.pack(side=LEFT)
        scroll = MiniScroller(frame, variable=self.month, min=1, max=12)
        scroll.pack(side=LEFT, fill=Y)
        frame = Frame(self.top)
        frame.pack(fill=X)
        label = Label(frame, text="Layout", width=6)
        label.pack(side=LEFT)
        values = tuple(map(lambda x: x[0], LAYOUTS))
        optionmenu = apply(OptionMenu, (frame, self.layout) + values)
        #optionmenu.config(width=reduce(max, map(len, values)))
        optionmenu.pack(side=LEFT, fill=X, expand=1)
        font_name = DefaultTextProperties().font.PostScriptName()
        frame = Frame(self.top)
        frame.pack(fill=X)
        label = Label(frame, text="Font", width=6)
        label.pack(side=LEFT)
        label = Label(frame, text=font_name, justify=RIGHT)
        label.pack(side=LEFT)
        frame = Frame(self.top)
        frame.pack(fill=X)
        label = Label(frame, text="Size", width=6)
        label.pack(side=LEFT)
        entry = Entry(frame, textvariable=self.size, width=6, justify=RIGHT)
        entry.pack(side=LEFT)
        scroll = MiniScroller(frame, variable=self.size, min=0)
        scroll.pack(side=LEFT, fill=Y)
        frame = Frame(self.top)
        frame.pack(fill=X)
        label = Label(frame, text="Grid W", width=6)
        label.pack(side=LEFT)
        entry = Entry(frame, textvariable=self.hspace, width=6, justify=RIGHT)
        entry.pack(side=LEFT)
        scroll = MiniScroller(frame, variable=self.hspace, min=0)
        scroll.pack(side=LEFT, fill=Y)
        label = Label(frame, text=" H")
        label.pack(side=LEFT)
        entry = Entry(frame, textvariable=self.vspace, width=6, justify=RIGHT)
        entry.pack(side=LEFT)
        scroll = MiniScroller(frame, variable=self.vspace, min=0)
        scroll.pack(side=LEFT, fill=Y)
        frame = Frame(self.top)
        frame.pack(fill=X)
        label = Label(frame, text="Align", width=6)
        label.pack(side=LEFT)
        values = tuple(map(lambda x: x[0], ALIGNMENTS))
        optionmenu = apply(OptionMenu, (frame, self.align) + values)
        optionmenu.pack(side=LEFT, fill=X, expand=1)
        frame = Frame(self.top)
        frame.pack(side=BOTTOM, fill=X)
	button = Button(frame, text=" OK ", command=self.ok)
	button.pack(side=LEFT)
	button = Button(frame, text="Cancel", command=self.cancel)
	button.pack(side=RIGHT)
    def get_parameters(self, flag=None):
        layout_name = self.layout.get()
        year = self.year.get()
        month = self.month.get()
        size = self.size.get()
        hspace = self.hspace.get()
        vspace = self.vspace.get()
        align_name = self.align.get()
        for name, layout_class in LAYOUTS:
            if name == layout_name:
                break
        else:
            raise RuntimeError, "should not reach here"
        for name, align in ALIGNMENTS:
            if name == align_name:
                break
        else:
            raise RuntimeError, "should not reach here"
        if flag is not None:
            global params
            params = (layout_name, year, month, size, hspace, vspace, align_name)
        return (layout_class, year, month, size, hspace, vspace, align)
    def ok(self, *args):
	self.close_dlg(self.get_parameters(1))

def calendar(context):
    args = ParameterDialog(context.application.root).RunDialog()
    if args is None:
        return
    layout_class, year, month, size, hspace, vspace, align = args
    layout = layout_class(year, month, size, hspace, vspace, align)
    context.main_window.PlaceObject(group.Group(layout.create()))

import Sketch.Scripting
Sketch.Scripting.AddFunction("Calendar", "Calendar", calendar,
                             script_type=Sketch.Scripting.AdvancedScript)
